### Cyclic Redundancy Codes (CRCs)

Daca reprezentam datele transmise ca pe un numar, atunci restul impartirii este valoarea pe care o putem introduce in header, iar receptorul poate verifica daca datele primite au acelasi rest.

```
De exemplu, atat transmitatorul cat si receptorul sunt de acord sa foloseasca
13131 ca si impartitor.
REST = 123131(PAYLOAD) % 13131 = 200

|DELIM|PAYLOAD|REST|DELIM|

```

In practica, nu folosim numere ci polinoame, printre altere fiind mult mai usor de lucrat cu ele (nu o sa avem carry). **Cyclic Redundancy Codes (CRC) reprezinta restul
impartirii polinomiale modulo 2 a datelor pe care vrem sa le trimitem**. 
Putem vedea payload-ul ca si reprezentarea unui polinom.

```
PAYLOAD=    'H'      'i'     '!'
          01001000 01101001 00100001
```

Cu reprezentarea matematica

\\( x^{22} + x^{19} + x^{14} + x^{13} + x^{11} +x^{8} +x^{5} + x^{0} \\)

De ce modulo 2 (inelul claselor de resturi modulo 2)? Deoarce vrem ca
indicii in urma calculelor sa fie 1 sau 0, altfel, atunci cand facem impartirea,
am ajunge la valori reale, iar noi, putem folosi doar valori binare.

Pentru optimizari, operatiile in
acest inel sunt echivalente cu XOR (e.g. 1 + 1 = 0).  In functie de polinomul la
care o sa ne raportam, avem diferite implementari de CRC.
[Aici](https://en.wikipedia.org/wiki/Cyclic_redundancy_check#Polynomial_representations_of_cyclic_redundancy_checks)
gasiti mai multe exemple de polinoame si la ce sunt folosite. CRC 32 foloseste
urmatorul polinom:

\\( x^{32} + x^{26} + x^{23} + x^{22} + x^{16} + x^{12} + x^{11} + x^{10} + x^{8} + x^{7} + x^{5} + x^{4} + x^{2} + x + 1 \\)

cu reprezentarea in binar si hexa:
```
0xEDB88320 	11101101-10111000_10000011_00100000
            x^0
```
Cum avem doar 32 de biti, nu ne intereseaza indicele lui x^32. Polinomul a fost ales astfel incat sa functioneze bine in cazul erorilor in rafala.

Pentru string-ul `"123456789"`, valoarea CRC32 este `0xFC891918`.

Un exemplu de imartire de polinoame modulo 2 este acesta:

```
10011 ) 11010110110000 = Bits of payload
=Poly   10011,,.,,....
        -----,,.,,....
         10011,.,,....  (operatia de xor cand primul bit e 1)
         10011,.,,....
         -----,.,,....
          00001.,,....  (cand primul bit e zero, doar face un shift stanga
          00000.,,....   pentru a lua urmatorul indice de exponent)
          -----.,,....
           00010,,....
           00000,,....
           -----,,....
            00101,....
            00000,....
            -----,....
             01011....
             00000....
             -----....
              10110...
              10011...
              -----...
               01010..
               00000..
               -----..
                10100.
                10011.
                -----.
                 01110
                 00000
                 -----
                  1110 = Remainder = The CRC!
```

O posibila implementare a algoritmului CRC32 urmatoarea:

```C
/* unsigned char *buffer contine payload-ul, len este lungimea agestuia */
/* Prin conventie crc-ul initial are toti bitii setati pe 1 */
uint32_t crc = -1; // 0xffffffff
const uint32_t POLY = 0xEDB88320;

/* Parcurgem fiecare byte din buffer */
while(len--)
{
    /* crc contine restul impartirii la fiecare etapa */
    /* nu ne intereseaza catul */
    /* adunam urmatorii 8 bytes din buffer */
    crc = crc ^ *buffer++;
    for( int bit = 0; bit < 8; bit++ )
    {
        /* 10011 ) 11010110110000 = Bytes of payload
            =Poly   10011,,.,,....
                    -----,,.,,....
                     10011,.,,....  (operatia de xor cand primul bit e 1)
                     10011,.,,....
                     -----,.,,....
                      00001.,,....  (asta e noua valoare a lui crc) (crc >> 1) ^ POLY
        */
        if( crc & 1 )
            crc = (crc >> 1) ^ POLY;
        else 
            /* 10011 ) 11010110110000 = Bytes of payload
                =Poly   10011,,.,,....
                        -----,,.,,....
                         10011,.,,....  
                         10011,.,,....
                         -----,.,,....
                          00001.,,....  primul bit e 0, 
                          00000.,,....  
                          -----.,,....
                           00010,,.... am facut shift la dreapta, pentru ca suntem pe **little endian**
            */
            crc = (crc >> 1);
    }
}
// Prin conventie, o sa facem flip la toti bitii
crc = ~crc;
```

---
**Note**

Pentru o intelegere mai buna a matematicii din spatele CRC, va recomandam [urmatorul video (40 min)](https://www.youtube.com/watch?v=izG7qT0EpBw).

---
