# IPv4

Protocolul IP este utitilzat pentru a permite dispozitivelor conectate in retele diferite sa
schimber informatii prin intermediul unui dispozitiv intermediar numit router. Header-ul unui
**pachet (packet)** IP este urmatorul:

```
+----+---------------+---------------+---------------+---------------+ 
|Word|       1       |       2       |       3       |       4       |
+----+---------------+---------------+---------------+---------------+
|Byte|0|1|2|3|4|5|6|7|8|9|0|1|2|3|4|5|6|7|8|9|0|1|2|3|4|5|6|7|8|9|0|1|
+----+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|   0|Version|  IHL  |Type of Service|          Total Length         |
+----+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|   4|         Identification        |Flags|      Fragment Offset    |
+----+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|   8|  Time to Live |    Protocol   |         Header Checksum       |
+----+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|  12|                       Source Address                          |
+----+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|  16|                    Destination Address                        |
+----+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
```

De notat faptul ca astazi nu vom atinge decat campurile `Time to Live`,
`checksum` si `adresa IP destinatie`. Campul `Time to Live` este un numar
decrementat de fiecare router pentru a evita buclele. `checksum` este campul
folosit pentru a verifica integritatea header-ului IP. `Destionation Address`
este adresa IP a destinatiei.

Urmatoarea strucura poate fi folosita pentru a reprezenta un pachet IPV4.
``` C
struct iphdr {
    // The following syntax means that version has 4 bits and ihl 4 bits.
    uint8_t    version:4,// don't care
        ihl:4;        // don't care
    uint8_t    tos;      // don't care
    uint16_t   tot_len;  // don't care
    uint16_t   id;       // don't care
    uint16_t   frag_off; // don't care
    uint8_t    ttl;      // Time to Live -> to avoid loops, we will decrement
    uint8_t    protocol; // don't care
    uint16_t   check;    // checksum     -> Since we modify TTL,
    // we need to recompute the checksum
    uint32_t   saddr;    // don't care
    uint32_t   daddr;    // the destination of the packet
};
```
Observam ca o adresa IP precum `10.30.4.2` poate fi reprezentata in memorie ca
un integer pe 32 de biti, `uint32_t`.

Campul `checksum` este complementul fata de 1 al sumei tuturor cuvintelor de 16
biti din header. Totusi, cum noi modificam doar campul `TTL` si pentru ca checksum
este o suma, exista o metoda mai rapida de a calcula noul checksum folosind
formula:

```
Fie:
HC  - vechiul checksum din header
C   - complementul fata de 1 al sumei campurilor din header
HC' - noul checksum
m   - vechea valoare a campului de 16 biti (TTL in cazul nostru)
m'  - moua valoare a campului de 16 biti (TTl --)

HC' = ~(C + (-m) + m') = HC + (m - m') = HC + m + ~m'
      complement fata
      de 1 al noului
        mesaj
```

Avem astfel o formula simpla de calculat noul checksum:

\\( NEW\\_CHECKSUM = OLD\\_CHECKSUM + OLD\\_TTL + \sim NEW\\_TTL) \\)
