# Mininet

Mininet foloseste python pentru a specifica o topologie. In urmatorul exemplu prezentam o topologie
formata din 3 dispozitive, dintre care unul actioneaza ca un router.

```python
from mininet.topo import Topo    
from mininet.cli import CLI    
from mininet.net import Mininet    
    
class MyTopo(Topo):        
    "O topologie este definita ca o clasa ce mosteneste Topo"        
        
    def build(self):        
        # Adaugam dispozitivele din topologie: hosts si un switch de L2     
        leftHost = self.addHost('host2')     
        rightHost = self.addHost('host1')
        # Pe acest dispozitiv, teoretic ar trebui pornit un program ce
        # implementeaza protocolul de rutare     
        router = self.addHost('router')    
        
        # Adaugam link-urile        
        self.addLink(leftHost, router)        
        self.addLink(rightHost, router)        
        
# Instantiem topologia        
topo = MyTopo()    
# Pornim simularea    
net = Mininet(topo)    
net.start()    
    
# Pornim terminalul de control al topologiei    
CLI(net)    
net.stop()
```

Vom folosi python pentru a rula topologia, presupunem ca topologia se afla in fisierul topo.py.

```bash
❯ sudo python3 topo.py
mininet> 
```

Din terminalul principal vom putea deschide terminale pe oricare dintre dispozitive:
``` Bash
# porneste un terminal pe host1
mininet> host1 xterm&
```

Comanda precedenta va deschide un terminal, din acel terminal vom putea executa orice
binar de linux care se afla si pe masina noastra, precum `Wireshark`.

```
sudo wireshark&
```

Sau pentru depanare `ping`:

```
❯ ping router_ip
PING router_ip (router_ip) 56(84) bytes of data.
64 bytes from host_ip: icmp_seq=1 ttl=58 time=21.2 ms
64 bytes from host_ip: icmp_seq=2 ttl=58 time=21.5 ms
64 bytes from host_ip: icmp_seq=3 ttl=58 time=20.6 ms
```

> Din terminalul unui dispozitiv putem rula ip address show pentru a vedea toate
interfetele disponibile si adresele IP de pe acestea.